import scraper.wrapper as _wrapper
import scraper.exceptions as _exceptions

# Public module attribues.
collection = _wrapper.collection
category = _wrapper.category
age = _wrapper.age
sort = _wrapper.sort

# Public module methods - wrappers around the original API.
def app(appId, lang='en', country='us', **kwargs):
    """
    Retrieves the full detail of an application.

    Parameters
    ----------
    appId : str
        The Google Play id of the application (the ?id= parameter on the url).
    lang : str, optional
        The two letter language code in which to fetch the app page (default is 'en').
    country : str, optional
        The two letter country code used to retrieve the applications (default is 'us').
    throttle : int, optional
        Upper bound to the amount of requests that will be attempted per second (default is None).
    **kwargs : dict
        Keyword arguments.

    Returns
    -------
    dict
        Dictionary containing the full details of an application.

    Raises
    ------
    ScraperException
        Raised if an error occured when scraping Google Play Store or parsing the response.
    """
    vargs = {
        'appId': appId,
        'lang': lang,
        'country': country,
        **kwargs
    }
    return _wrapper.app(**vargs)

def list(collection=None, category=None, age=None, num=500, lang='en', country='us', fullDetail=False, **kwargs):
    """
    Retrieves a list of applications from one of the collections at Google Play.

    Parameters
    ----------
    collection : str, optional
        The Google Play collection that will be retrieved (default is scraper.collection['TOP_FREE'])
    category : str, optional
        The app category to filter by (default is None).
    age : {None, scraper.age['FIVE_UNDER'], scraper.age['SIX_EIGHT'], scraper.age['NINE_UP']}, optional
        The age range to filter the apps (only for FAMILY and its subcategories; default is None).
    num : int, optional
        The number of apps to retrieve (default is 500).
    lang : str, optional
        The two letter language code in which to fetch the app page (default is 'en').
    country : str, optional
        The two letter country code used to retrieve the applications (default is 'us').
    fullDetail : bool, optional
        If True, an extra request will be made for every resulting app to fetch its full detail (default is False).
    throttle : int, optional
        Upper bound to the amount of requests that will be attempted per second (default is None).
    **kwargs : dict
        Keyword arguments.

    Returns
    -------
    list of dict
        List of dictionaries containing details of applications from one of the collections at Google Play.

    Raises
    ------
    ScraperException
        Raised if an error occured when scraping Google Play Store or parsing the response.
    """
    vargs = {
        'collection': _wrapper.collection['TOP_FREE'] if collection is None else collection,
        'category': category,
        'age': age,
        'num': num,
        'lang': lang,
        'country': country,
        'fullDetail': fullDetail,
        **kwargs
    }
    return _wrapper.list(**vargs)

def search(term, num=20, lang='en', country='us', fullDetail=False, price='all', **kwargs):
    """
    Retrieves a list of apps that results from searching by the given term.

    Parameters
    ----------
    term : str
        The term to search by.
    num : int, optional
        The number of apps to retrieve (default is 20, max is 250).
    lang : str, optional
        The two letter language code in which to fetch the app page (default is 'en').
    country : str, optional
        The two letter country code used to retrieve the applications (default is 'us').
    fullDetail : bool, optional
        If True, an extra request will be made for every resulting app to fetch its full detail (default is False).
    price : {'all', 'free', 'paid'}, optional
        Allows to control if the results apps are free, paid or both (default is 'all').
    throttle : int, optional
        Upper bound to the amount of requests that will be attempted per second (default is None).
    **kwargs : dict
        Keyword arguments.

    Returns
    -------
    list of dict
        List of dictionaries containing details of applications that result from searching by the given term.

    Raises
    ------
    ScraperException
        Raised if an error occured when scraping Google Play Store or parsing the response.
    """
    vargs = {
        'term': term,
        'num': num,
        'lang': lang,
        'country': country,
        'fullDetail': fullDetail,
        'price': price,
        **kwargs
    }
    return _wrapper.search(**vargs)

def developer(devId, lang='en', country='us', num=60, fullDetail=False, **kwargs):
    """
    Returns the list of applications by the given developer name.

    Parameters
    ----------
    devId : str
        The name of the developer.
    lang : str, optional
        The two letter language code in which to fetch the app page (default is 'en').
    country : str, optional
        The two letter country code used to retrieve the applications (default is 'us').
    num : int, optional
        The number of apps to retrieve (default is 60).
    fullDetail : bool, optional
        If True, an extra request will be made for every resulting app to fetch its full detail (default is False).
    throttle : int, optional
        Upper bound to the amount of requests that will be attempted per second (default is None).
    **kwargs : dict
        Keyword arguments.

    Returns
    -------
    list of dict
        List of dictionaries containing details of applications by the given developer name.

    Raises
    ------
    ScraperException
        Raised if an error occured when scraping Google Play Store or parsing the response.
    """
    vargs = {
        'devId': devId,
        'lang': lang,
        'country': country,
        'num': num,
        'fullDetail': fullDetail,
        **kwargs
    }
    return _wrapper.developer(**vargs)

def suggest(term, lang='en', country='us', **kwargs):
    """
    Given a string returns up to five suggestion to complete a search query term.

    Parameters
    ----------
    term : str
        The term to get suggestions for.
    lang : str, optional
        The two letter language code in which to fetch the app page (default is 'en').
    country : str, optional
        The two letter country code used to retrieve the applications (default is 'us').
    throttle : int, optional
        Upper bound to the amount of requests that will be attempted per second (default is None).
    **kwargs : dict
        Keyword arguments.

    Returns
    -------
    list of str
        List of suggestions to complete the given search query term.

    Raises
    ------
    ScraperException
        Raised if an error occured when scraping Google Play Store or parsing the response.
    """
    vargs = {
        'term': term,
        'lang': lang,
        'country': country,
        **kwargs
    }
    return _wrapper.suggest(**vargs)

def reviews(appId, lang='en', country='us', sort=None, num=100, paginate=False, nextPaginationToken=None, **kwargs):
    """
    Retrieves a page of reviews for a specific application.

    Parameters
    ----------
    appId : str
        The Google Play id of the application (the ?id= parameter on the url).
    lang : str, optional
        The two letter language code in which to fetch the app page (default is 'en').
    country : str, optional
        The two letter country code used to retrieve the applications (default is 'us').
    sort : {scraper.sort['NEWEST'], scraper.sort['RATING'], scraper.sort['HELPFULNESS']}, optional
        The way the reviews are going to be sorted (default is scraper.sort['NEWEST']).
    num : int, optional
        Number of reviews to be captured (default is 100).
    paginate : bool, optional
        Defines if the result will be paginated (default is False).
    nextPaginationToken : str
        The next token to paginate (default is None).
    throttle : int, optional
        Upper bound to the amount of requests that will be attempted per second (default is None).
    **kwargs : dict
        Keyword arguments.

    Returns
    -------
    list of dict
        List of dictionaries containing the reviews and the nextPaginationToken.

    Raises
    ------
    ScraperException
        Raised if an error occured when scraping Google Play Store or parsing the response.
    """
    vargs = {
        'appId': appId,
        'lang': lang,
        'country': country,
        'sort': _wrapper.sort['NEWEST'] if sort is None else sort,
        'num': num,
        'paginate': paginate,
        'nextPaginationToken': nextPaginationToken,
        **kwargs
    }
    return _wrapper.reviews(**vargs)

def similar(appId, lang='en', country='us', fullDetail=False, **kwargs):
    """
    Returns a list of similar apps to the one specified.

    Parameters
    ----------
    appId : str
        The Google Play id of the application to get similar apps for.
    lang : str, optional
        The two letter language code in which to fetch the app page (default is 'en').
    country : str, optional
        The two letter country code used to retrieve the applications (default is 'us').
    fullDetail : bool, optional
        If True, an extra request will be made for every resulting app to fetch its full detail (default is False).
    throttle : int, optional
        Upper bound to the amount of requests that will be attempted per second (default is None).
    **kwargs : dict
        Keyword arguments.

    Returns
    -------
    list of dict
        List of dictionaries containing details of applications similar to the one specified.

    Raises
    ------
    ScraperException
        Raised if an error occured when scraping Google Play Store or parsing the response.
    """
    vargs = {
        'appId': appId,
        'lang': lang,
        'country': country,
        'fullDetail': fullDetail,
        **kwargs
    }
    return _wrapper.similar(**vargs)

def permissions(appId, lang='en', short=False, **kwargs):
    """
    Returns the list of permissions an app has access to.

    Parameters
    ----------
    appId : str
        The Google Play id of the application to get permissions for.
    lang : str, optional
        The two letter language code in which to fetch the app page (default is 'en').
    short : str, optional
        If True, the permission names will be returned instead of permission/description objects (default is False).
    throttle : int, optional
        Upper bound to the amount of requests that will be attempted per second (default is None).
    **kwargs : dict
        Keyword arguments.

    Returns
    -------
    list of dict or list of str
        List of dictionaries (if `short` is True) or list of strings (if `short` is False)
        corresponding to the permissions an app has access to.

    Raises
    ------
    ScraperException
        Raised if an error occured when scraping Google Play Store or parsing the response.
    """
    vargs = {
        'appId': appId,
        'lang': lang,
        'short': short,
        **kwargs
    }
    return _wrapper.permissions(**vargs)

def categories(**kwargs):
    """
    Retrieve a full list of categories present from dropdown menu on Google Play Store.

    Parameters
    ----------
    **kwargs : dict
        Keyword arguments.

    Returns
    -------
    list of str
        List containing available categories from the dropdown menu on Google Play Store.

    Raises
    ------
    ScraperException
        Raised if an error occured when scraping Google Play Store or parsing the response.
    """
    return _wrapper.categories(**kwargs)



import subprocess
import json
import os
import logging

from .exceptions import ScraperException

SELF_DIR = os.path.dirname(os.path.abspath(__file__))
logger = logging.getLogger('__main__')

# Private module class.
class _Wrapper:

    require_dir = os.path.join(SELF_DIR, 'node_modules', 'google-play-scraper')

    api_script = (
        "var gplay = require('{}'){};"
        "gplay.{}({{{}}})"
        ".then(JSON.stringify).then(console.log).catch(console.log);"
    )

    var_script = (
        "var gplay = require('{}'){};"
        "let x = gplay.{};"
        "console.log(JSON.stringify(x));"
    )

    def __init__(self, memoization=False, vars=[]):
        self.memoization = '.memoized()' if memoization else ''
        for var in vars:
            setattr(_Wrapper, var, self._execute_var(var))

    def _execute_api(self, fn_name, keys, **kwargs):
        cmd = self._get_args(keys, **kwargs)
        script = self.api_script.format(self.require_dir, self.memoization, fn_name, cmd)
        return self._execute(script)

    def _execute_var(self, var_name):
        script = self.var_script.format(self.require_dir, self.memoization, var_name)
        return self._execute(script)

    def _execute(self, script):
        logger.debug('Executing script: {}'.format(script))
        args = ['node', '-e', script]
        try:
            process = subprocess.run(args, capture_output=True, check=True)
        except subprocess.CalledProcessError as e:
            stderr = e.stderr.decode()
            raise ScraperException(stderr) from None
        stdout = process.stdout.decode()
        try:
            return json.loads(stdout)
        except json.decoder.JSONDecodeError as e:
            raise ScraperException(stdout) from None

    def _get_args(self, keys, **kwargs):
        def stringify(x):
            if x is None:
                return 'null'
            if isinstance(x, str):
                return "'{}'".format(x)
            elif type(x) == bool:
                return '{}'.format(str(x).lower())
            return '{}'.format(x)

        args = ', '.join(["{}: {}".format(k, stringify(v)) for k, v in kwargs.items() if k in keys + ['throttle']])
        return args

# Private module attributes.
_wrapper = _Wrapper(vars=['collection', 'category', 'age', 'sort'])

# Public module attributes.
collection = _wrapper.collection
category = _wrapper.category
age = _wrapper.age
sort = _wrapper.sort

# Public module methods.
def app(appId, **kwargs):
    keys = ['appId', 'lang', 'country']
    output = _wrapper._execute_api('app', keys, **{'appId': appId, **kwargs})
    return output

def list(**kwargs):
    keys = [
        'collection', 'category', 'age', 'num',
        'lang', 'country', 'fullDetail'
    ]
    output = _wrapper._execute_api('list', keys, **kwargs)
    return output

def search(term, **kwargs):
    keys = [
        'term', 'num', 'lang',
        'country', 'fullDetail', 'price'
    ]
    output = _wrapper._execute_api('search', keys,  **{'term': term, **kwargs})
    return output

def developer(devId, **kwargs):
    keys = ['devId', 'lang', 'country', 'num', 'fullDetail']
    output = _wrapper._execute_api('developer', keys,  **{'devId': devId, **kwargs})
    return output

def suggest(term, **kwargs):
    keys = ['term', 'lang', 'country']
    output = _wrapper._execute_api('suggest', keys,  **{'term': term, **kwargs})
    return output

def reviews(appId, **kwargs):
    keys = [
        'appId', 'lang', 'country', 'sort',
        'num', 'paginate', 'nextPaginationToken'
    ]
    output = _wrapper._execute_api('reviews', keys,  **{'appId': appId, **kwargs})
    return output

def similar(appId, **kwargs):
    keys = ['appId', 'lang', 'country', 'fullDetail']
    output = _wrapper._execute_api('similar', keys,  **{'appId': appId, **kwargs})
    return output

def permissions(appId, **kwargs):
    keys = ['appId', 'lang', 'short']
    output = _wrapper._execute_api('permissions', keys,  **{'appId': appId, **kwargs})
    return output

def categories(**kwargs):
    output = _wrapper._execute_api('categories', [],  **kwargs)
    return output



import os
from setuptools import setup, find_packages

# The directory containing this file.
WORK_DIR = os.path.dirname(os.path.abspath(__file__))

# The text of the README file.
README = os.path.join(WORK_DIR, 'README.md')
with open(README, 'r') as f:
    long_description = f.read()

setup(
    name='google-play-scraper-py',
    version='0.2.3',
    description='Scrape application data from the Google Play store.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/blues-lab/google-play-scraper-py',
    author='Berkeley Lab for Usable and Experimental Security (BLUES)',
    author_email='blues@berkeley.edu',
    license='MIT',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.9'
    ],
    packages=find_packages(),
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'scraper=scraper.__main__:main',
        ]
    },
)



#!/usr/bin/env python3
import os
import sys
import json
import logging
import argparse
import scraper.scraper

logger = logging.getLogger(__name__)

def _setup_logger():
    filename = 'output.log'
    datefmt = '%Y-%m-%d %H:%M:%S'
    log_format = '[%(asctime)s] - %(levelname)-8s %(message)s'
    formatter = logging.Formatter(log_format, datefmt)

    file_handler = logging.FileHandler(filename, 'w')
    file_handler.setFormatter(formatter)
    logger.addHandler(file_handler)

    stdout_handler = logging.StreamHandler(sys.stdout)
    stdout_handler.setFormatter(formatter)
    logger.addHandler(stdout_handler)
    logger.setLevel(logging.INFO)

def _setup_parsers():
    # Create the top-level parser.
    parser, subparsers = argparse.ArgumentParser(), {}
    handle = parser.add_subparsers(help='Command to pass to the Google Play Store scraper.', dest='command')

    # Create the parser for the 'app' command.
    subparsers['app'] = handle.add_parser('app', help='Retrieves the full detail of an application.')
    subparsers['app'].add_argument('appId', help='the Google Play id of the application (the ?id= parameter on the url).')
    subparsers['app'].add_argument('--lang', help='(optional, defaults to \'en\'): the two letter language code in which to fetch the app page.')
    subparsers['app'].add_argument('--country', help='(optional, defaults to \'us\'): the two letter country code used to retrieve the applications.')

    # Create the parser for the 'list' command.
    subparsers['list'] = handle.add_parser('list', help='Retrieves a list of applications from one of the collections at Google Play.')
    subparsers['list'].add_argument('--collection', help='(optional, defaults to collection.TOP_FREE): the Google Play collection that will be retrieved.')
    subparsers['list'].add_argument('--category', help='(optional, defaults to no category): the app category to filter by.')
    subparsers['list'].add_argument('--age', help='(optional, defaults to no age filter): the age range to filter the apps (only for FAMILY and its subcategories). Available options are age.FIVE_UNDER, age.SIX_EIGHT, age.NINE_UP.')
    subparsers['list'].add_argument('--num', help='(optional, defaults to 500): the amount of apps to retrieve.')
    subparsers['list'].add_argument('--lang', help='(optional, defaults to \'en\'): the two letter language code used to retrieve the applications.')
    subparsers['list'].add_argument('--country', help='(optional, defaults to \'us\'): the two letter country code used to retrieve the applications.')
    subparsers['list'].add_argument('--fullDetail', action='store_true', help='(optional, defaults to false): if true, an extra request will be made for every resulting app to fetch its full detail.')

    # Create the parser for the 'search' command.
    subparsers['search'] = handle.add_parser('search', help='Retrieves a list of apps that results of searching by the given term.')
    subparsers['search'].add_argument('term', help='the term to search by.')
    subparsers['search'].add_argument('--num', help='(optional, defaults to 20, max is 250): the amount of apps to retrieve.')
    subparsers['search'].add_argument('--lang', help='(optional, defaults to \'en\'): the two letter language code used to retrieve the applications.')
    subparsers['search'].add_argument('--country', help='(optional, defaults to \'us\'): the two letter country code used to retrieve the applications.')
    subparsers['search'].add_argument('--fullDetail', action='store_true', help='(optional, defaults to false): if true, an extra request will be made for every resulting app to fetch its full detail.')
    subparsers['search'].add_argument('--price', help='(optional, defaults to all): allows to control if the results apps are free, paid or both. Accepted values are: \'all\', \'free\' and \'paid\'.')

    # Create the parser for the 'developer' command.
    subparsers['developer'] = handle.add_parser('developer', help='Returns the list of applications by the given developer name.')
    subparsers['developer'].add_argument('devId', help='the name of the developer.')
    subparsers['developer'].add_argument('--lang', help='(optional, defaults to \'en\'): the two letter language code in which to fetch the app list.')
    subparsers['developer'].add_argument('--country', help='(optional, defaults to \'us\'): the two letter country code used to retrieve the applications. Needed when the app is available only in some countries.')
    subparsers['developer'].add_argument('--num', help='(optional, defaults to 60): the amount of apps to retrieve.')
    subparsers['developer'].add_argument('--fullDetail', action='store_true', help='(optional, defaults to false): if true, an extra request will be made for every resulting app to fetch its full detail.')

    # Create the parser for the 'suggest' command.
    subparsers['suggest'] = handle.add_parser('suggest', help='Given a string returns up to five suggestion to complete a search query term.')
    subparsers['suggest'].add_argument('term', help='the term to get suggestions for.')
    subparsers['suggest'].add_argument('--lang', help='(optional, defaults to \'en\'): the two letter language code used to retrieve the suggestions.')
    subparsers['suggest'].add_argument('--country', help='(optional, defaults to \'us\'): the two letter country code used to retrieve the suggestions.')

    # Create the parser for the 'reviews' command.
    subparsers['reviews'] = handle.add_parser('reviews', help='Retrieves a page of reviews for a specific application.')
    subparsers['reviews'].add_argument('appId', help='Unique application id for Google Play. (e.g. id=com.mojang.minecraftpe maps to Minecraft: Pocket Edition game).')
    subparsers['reviews'].add_argument('--lang', help='(optional, defaults to \'en\'): the two letter language code in which to fetch the reviews.')
    subparsers['reviews'].add_argument('--country', help='(optional, defaults to \'us\'): the two letter country code in which to fetch the reviews.')
    subparsers['reviews'].add_argument('--sort', help='(optional, defaults to sort.NEWEST): The way the reviews are going to be sorted. Accepted values are: sort.NEWEST, sort.RATING and sort.HELPFULNESS.')
    subparsers['reviews'].add_argument('--num', help='(optional, defaults to 100): Quantity of reviews to be captured.')
    subparsers['reviews'].add_argument('--paginate', help='(optional, defaults to false): Defines if the result will be paginated')
    subparsers['reviews'].add_argument('--nextPaginationToken', help='(optional, defaults to null): The next token to paginate')

    # Create the parser for the 'similar' command.
    subparsers['similar'] = handle.add_parser('similar', help='Returns a list of similar apps to the one specified.')
    subparsers['similar'].add_argument('appId', help='the Google Play id of the application to get similar apps for.')
    subparsers['similar'].add_argument('--lang', help='(optional, defaults to \'en\'): the two letter language code used to retrieve the applications.')
    subparsers['similar'].add_argument('--country', help='(optional, defaults to \'us\'): the two letter country code used to retrieve the applications.')
    subparsers['similar'].add_argument('--fullDetail', action='store_true', help='(optional, defaults to false): if true, an extra request will be made for every resulting app to fetch its full detail.')

    # Create the parser for the 'permissions' command.
    subparsers['permissions'] = handle.add_parser('permissions', help='Returns the list of permissions an app has access to.')
    subparsers['permissions'].add_argument('appId', help='the Google Play id of the application to get permissions for.')
    subparsers['permissions'].add_argument('--lang',  help='(optional, defaults to \'en\'): the two letter language code in which to fetch the permissions.')
    subparsers['permissions'].add_argument('--short', action='store_true', help='(optional, defaults to false): if true, the permission names will be returned instead of permission/description objects.')

    # Create the parser for the 'categories' command.
    subparsers['categories'] = handle.add_parser('categories', help='Retrieve a full list of categories present from dropdown menu on Google Play.')

    # Add universal commands to all subparsers.
    for subparser in subparsers.values():
        subparser.add_argument('--output_path', '-o', help='')
        subparser.add_argument('--throttle', default=1, type=int, help='Upper bound to the amount of requests that will be attempted per second.')
        subparser.add_argument('--verbose', '-v', action='store_true', help='Enable verbose logging')

    return parser, subparsers

class CommandLineTool:

    def __init__(self, parser):
        args = parser.parse_args()

        wrapper_fns = [x for x in dir(scraper) if callable(getattr(scraper, x))]
        wrapper_fns = [x for x in wrapper_fns if len(x) > 0 and x[0] != '_']
        vargs = {k:v for k, v in vars(args).items() if v is not None}

        # Check if the supplied command is valid.
        if args.command is None or not hasattr(scraper, args.command):
            print('Unrecognized command: {}.'.format(args.command))
            parser.print_help()
            exit(1)

        logger.setLevel(logging.DEBUG) if args.verbose else logger.setLevel(logging.INFO)
        results = getattr(scraper, args.command)(**vargs)

        # Save output to file if provided or print to STDOUT.
        print(results) if args.output_path is None else self._save_results(results, args.output_path)

    def _save_results(self, results, output_path):
        with open(output_path, 'w') as f:
            json.dump(results, f)

def main():
    _setup_logger()
    parser, subparsers = _setup_parsers()
    CommandLineTool(parser)

if __name__ == "__main__":
    main()
